@@ -242,10 +242,10 @@ void WindowThumbnail::iconToTexture(WindowTextureNode *textureNode)
 {
     QIcon icon;
     if (KWindowSystem::self()->hasWId(m_winId)) {
-        icon = KWindowSystem::self()->icon(m_winId);
+        icon = KWindowSystem::self()->icon(m_winId, boundingRect().width(), boundingRect().height());
     } else {
         // fallback to plasma icon
-        icon = QIcon::fromTheme("plasma");
+        icon = QIcon::fromTheme(QStringLiteral("plasma"));
     }
     QImage image = icon.pixmap(boundingRect().size().toSize()).toImage();
     textureNode->reset(window()->createTextureFromImage(image));
@@ -513,7 +513,7 @@ bool WindowThumbnail::loadGLXTexture()
     // As the GLXFBConfig might be context specific and we cannot be sure
     // that the code might be entered from different contexts, the cache
     // also maps the cached configs against the context.
-    static QMap<GLXContext, QMap<int, GLXFBConfig> > s_fbConfigs;
+    static QHash<GLXContext, QMap<int, GLXFBConfig> > s_fbConfigs;
     auto it = s_fbConfigs.find(glxContext);
     if (it == s_fbConfigs.end()) {
         // create a map entry for the current context
@@ -584,6 +584,7 @@ void WindowThumbnail::stopRedirecting()
         return;
     }
     xcb_composite_unredirect_window(c, m_winId, XCB_COMPOSITE_REDIRECT_AUTOMATIC);
+    discardPixmap();
     if (m_damage == XCB_NONE) {
         return;
     }
