@@ -37,7 +37,6 @@
 #include <QDebug>
 #include <kauthorized.h>
 #include <klocalizedstring.h>
-#include <kservicetypetrader.h>
 #include <KConfigSkeleton>
 #include <KConfigLoader>
 
@@ -50,6 +49,7 @@
 #include "containmentactions.h"
 #include "corona.h"
 #include "pluginloader.h"
+#include "debug_p.h"
 
 #include "private/applet_p.h"
 
@@ -90,6 +90,7 @@ Containment::Containment(const QString &packagePath, uint appletId)
 
 Containment::~Containment()
 {
+    qDeleteAll(d->localActionPlugins);
     delete d;
 }
 
@@ -100,13 +101,13 @@ void Containment::init()
     if (d->type == Types::NoContainmentType) {
         //setContainmentType(Plasma::Types::DesktopContainment);
         //Try to determine the containment type. It must be done as soon as possible
-        QString type = pluginInfo().property("X-Plasma-ContainmentType").toString();
+        QString type = pluginInfo().property(QStringLiteral("X-Plasma-ContainmentType")).toString();
 
-        if (type == "Panel") {
+        if (type == QLatin1String("Panel")) {
             setContainmentType(Plasma::Types::PanelContainment);
-        } else if (type == "Custom") {
+        } else if (type == QLatin1String("Custom")) {
             setContainmentType(Plasma::Types::CustomContainment);
-        } else if (type == "CustomPanel") {
+        } else if (type == QLatin1String("CustomPanel")) {
             setContainmentType(Plasma::Types::CustomPanelContainment);
             //default to desktop
         } else {
@@ -120,17 +121,17 @@ void Containment::init()
 
     //fix the text of the actions that need title()
     //btw, do we really want to use title() when it's a desktopcontainment?
-    QAction *closeApplet = actions()->action("remove");
+    QAction *closeApplet = actions()->action(QStringLiteral("remove"));
     if (closeApplet) {
         closeApplet->setText(i18nc("%1 is the name of the applet", "Remove this %1", title()));
     }
 
-    QAction *configAction = actions()->action("configure");
+    QAction *configAction = actions()->action(QStringLiteral("configure"));
     if (configAction) {
         configAction->setText(i18nc("%1 is the name of the applet", "%1 Settings", title()));
     }
 
-    QAction *appletBrowserAction = actions()->action("add widgets");
+    QAction *appletBrowserAction = actions()->action(QStringLiteral("add widgets"));
     if (appletBrowserAction) {
         appletBrowserAction->setVisible(unlocked);
         appletBrowserAction->setEnabled(unlocked);
@@ -138,10 +139,10 @@ void Containment::init()
     }
 
     if (immutability() != Types::SystemImmutable && corona()) {
-        QAction *lockDesktopAction = corona()->actions()->action("lock widgets");
+        QAction *lockDesktopAction = corona()->actions()->action(QStringLiteral("lock widgets"));
         //keep a pointer so nobody notices it moved to corona
         if (lockDesktopAction) {
-            actions()->addAction("lock widgets", lockDesktopAction);
+            actions()->addAction(QStringLiteral("lock widgets"), lockDesktopAction);
         }
     }
 
@@ -166,11 +167,11 @@ void Containment::restore(KConfigGroup &group)
 {
     /*
     #ifndef NDEBUG
-    // qDebug() << "!!!!!!!!!!!!initConstraints" << group.name() << d->type;
-    // qDebug() << "    location:" << group.readEntry("location", (int)d->location);
-    // qDebug() << "    geom:" << group.readEntry("geometry", geometry());
-    // qDebug() << "    formfactor:" << group.readEntry("formfactor", (int)d->formFactor);
-    // qDebug() << "    screen:" << group.readEntry("screen", d->screen);
+    // qCDebug(LOG_PLASMA) << "!!!!!!!!!!!!initConstraints" << group.name() << d->type;
+    // qCDebug(LOG_PLASMA) << "    location:" << group.readEntry("location", (int)d->location);
+    // qCDebug(LOG_PLASMA) << "    geom:" << group.readEntry("geometry", geometry());
+    // qCDebug(LOG_PLASMA) << "    formfactor:" << group.readEntry("formfactor", (int)d->formFactor);
+    // qCDebug(LOG_PLASMA) << "    screen:" << group.readEntry("screen", d->screen);
     #endif
     */
     setLocation((Plasma::Types::Location)group.readEntry("location", (int)d->location));
@@ -189,10 +190,10 @@ void Containment::restore(KConfigGroup &group)
         KConfigGroup cfg = KConfigGroup(corona()->config(), "ActionPlugins");
         cfg = KConfigGroup(&cfg, QString::number(containmentType()));
 
-        //qDebug() << cfg.keyList();
+        //qCDebug(LOG_PLASMA) << cfg.keyList();
         if (cfg.exists()) {
             foreach (const QString &key, cfg.keyList()) {
-                //qDebug() << "loading" << key;
+                //qCDebug(LOG_PLASMA) << "loading" << key;
                 setContainmentActions(key, cfg.readEntry(key, QString()));
             }
         } else { //shell defaults
@@ -213,7 +214,7 @@ void Containment::restore(KConfigGroup &group)
 
     /*
     #ifndef NDEBUG
-    // qDebug() << "Containment" << id() <<
+    // qCDebug(LOG_PLASMA) << "Containment" << id() <<
     #endif
                 "screen" << screen() <<
                 "geometry is" << geometry() <<
@@ -259,11 +260,15 @@ void Containment::restoreContents(KConfigGroup &group)
 {
     KConfigGroup applets(&group, "Applets");
 
+    //restore the applets ordered by id
+    QStringList groups = applets.groupList();
+    qSort(groups.begin(), groups.end());
+
     // Sort the applet configs in order of geometry to ensure that applets
     // are added from left to right or top to bottom for a panel containment
     QList<KConfigGroup> appletConfigs;
-    foreach (const QString &appletGroup, applets.groupList()) {
-        //qDebug() << "reading from applet group" << appletGroup;
+    foreach (const QString &appletGroup, groups) {
+        //qCDebug(LOG_PLASMA) << "reading from applet group" << appletGroup;
         KConfigGroup appletConfig(&applets, appletGroup);
         appletConfigs.append(appletConfig);
     }
@@ -281,6 +286,12 @@ void Containment::restoreContents(KConfigGroup &group)
 
         d->createApplet(plugin, QVariantList(), appId);
     }
+
+    foreach (Applet *applet, Containment::applets()) {
+        if (!applet->pluginInfo().isValid()) {
+            applet->updateConstraints(Plasma::Types::UiReadyConstraint);
+        }
+    }
 }
 
 Plasma::Types::ContainmentType Containment::containmentType() const
@@ -319,7 +330,7 @@ void Containment::setFormFactor(Types::FormFactor formFactor)
         return;
     }
 
-    //qDebug() << "switching FF to " << formFactor;
+    //qCDebug(LOG_PLASMA) << "switching FF to " << formFactor;
     d->formFactor = formFactor;
 
     updateConstraints(Plasma::Types::FormFactorConstraint);
@@ -352,25 +363,29 @@ void Containment::setLocation(Types::Location location)
 
 Applet *Containment::createApplet(const QString &name, const QVariantList &args)
 {
-    return d->createApplet(name, args);
+    Plasma::Applet *applet = d->createApplet(name, args);
+    if (applet) {
+        emit appletCreated(applet);
+    }
+    return applet;
 }
 
 void Containment::addApplet(Applet *applet)
 {
-    if (immutability() != Types::Mutable) {
+    if (!applet) {
+#ifndef NDEBUG
+        // qCDebug(LOG_PLASMA) << "adding null applet!?!";
+#endif
         return;
     }
 
-    if (!applet) {
-#ifndef NDEBUG
-        // qDebug() << "adding null applet!?!";
-#endif
+    if (immutability() != Types::Mutable && !applet->property("org.kde.plasma:force-create").toBool()) {
         return;
     }
 
     if (d->applets.contains(applet)) {
 #ifndef NDEBUG
-        // qDebug() << "already have this applet!";
+        // qCDebug(LOG_PLASMA) << "already have this applet!";
 #endif
     }
 
@@ -407,12 +422,16 @@ void Containment::addApplet(Applet *applet)
         applet->setParent(this);
     }
 
-    d->applets << applet;
+    //make sure the applets are sorted by id
+    auto position = std::lower_bound(d->applets.begin(), d->applets.end(), applet, [](Plasma::Applet *a1,  Plasma::Applet *a2) {
+        return a1->id() < a2->id();
+    });
+    d->applets.insert(position, applet);
 
-    if (!applet->d->uiReady) {
+    if (!d->uiReady) {
         d->loadingApplets << applet;
-        if (static_cast<Applet *>(this)->d->uiReady) {
-            static_cast<Applet *>(this)->d->uiReady = false;
+        if (d->uiReady) {
+            d->uiReady = false;
             emit uiReadyChanged(false);
         }
     }
@@ -493,13 +512,11 @@ void Containment::setContainmentActions(const QString &trigger, const QString &p
     KConfigGroup cfg = d->containmentActionsConfig();
     ContainmentActions *plugin = 0;
 
-    if (containmentActions().contains(trigger)) {
-        plugin = containmentActions().value(trigger);
-        if (plugin->pluginInfo().pluginName() != pluginName) {
-            containmentActions().remove(trigger);
-            delete plugin;
-            plugin = 0;
-        }
+    plugin = containmentActions().value(trigger);
+    if (plugin && plugin->pluginInfo().pluginName() != pluginName) {
+        containmentActions().remove(trigger);
+        delete plugin;
+        plugin = 0;
     }
 
     if (pluginName.isEmpty()) {
@@ -536,8 +553,7 @@ QHash<QString, ContainmentActions *> &Containment::containmentActions()
 
 bool Containment::isUiReady() const
 {
-    const Applet *a = static_cast<const Applet *>(this);
-    return a->d->uiReady && a->d->started;
+    return d->uiReady && d->appletsUiReady && Applet::d->started;
 }
 
 void Containment::setActivity(const QString &activityId)
@@ -563,13 +579,13 @@ void Containment::reactToScreenChange()
 {
     int newScreen = screen();
 
-    KConfigGroup c = config();
     if (newScreen >= 0) {
         d->lastScreen = newScreen;
+        KConfigGroup c = config();
         c.writeEntry("lastScreen", d->lastScreen);
+        emit configNeedsSaving();
     }
 
-    emit configNeedsSaving();
     emit screenChanged(newScreen);
 }
 
