@@ -33,6 +33,7 @@ class Svg;
 
 class SharedSvgRenderer : public QSvgRenderer, public QSharedData
 {
+    Q_OBJECT
 public:
     typedef QExplicitlySharedDataPointer<SharedSvgRenderer> Ptr;
 
@@ -63,10 +64,10 @@ public:
     ~SvgPrivate();
 
     //This function is meant for the rects cache
-    QString cacheId(const QString &elementId);
+    QString cacheId(const QString &elementId) const;
 
     //This function is meant for the pixmap cache
-    QString cachePath(const QString &path, const QSize &size);
+    QString cachePath(const QString &path, const QSize &size) const;
 
     bool setImagePath(const QString &imagePath);
 
@@ -111,6 +112,7 @@ public:
     unsigned int lastModified;
     qreal devicePixelRatio;
     qreal scaleFactor;
+    Svg::Status status;
     bool multipleImages : 1;
     bool themed : 1;
     bool useSystemColors : 1;
